---------------------------------------------------------------------------------
-- Module Name:    serIn - Behavioral 
-- Project Name: 		FirstHoughArchitectureBig 
-- Target Devices:   Virtex 5 FXT 130
-- Tool versions: ISE 12.1 
-- Description:--  Berechnet die generalisiert HT allerdings ohne Peak-Such im Houghraum. 
--  Als Eingabe kommt ein std_logic, welches die gltigen Pixel eines Bildes anzeigt. 
--  Die zu suchende Form ist als LUT gespeichert mit signed-Werten.
--  Als Ergebnis kommt ein 2D-Array (Hough_Space) als output-port welches nach Abbarbeitung des Testbildes den 
--  kompletten Hough-Raum mit allen Votes beinhaltet. 
--  Luft mit 2 Warnungen durch die Synthese
--------------------------------------------------------------------------------
--
-- *****************************************************************************
-- *
-- *               Automatically generated files for hough trafo
-- *                   File: Module serialIn for Hough TRansformation
-- *
-- *                                                                    V0.9 schfra
-- *****************************************************************************
--
-- date:     07/Sep/2011 13:19:11
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

library work; 
use work.testbild.ALL; 
use work.referenceData.ALL; 
use work.shape_data.ALL;

entity serIn is 
	port(
		clk : std_logic;
		reset : in std_logic;
		serialIn : std_logic;
		hough_space : out hough_array;
		enable : in std_logic
	);
end serIn;

architecture Behavioral of serIn is 
--Signals
signal sig_colcounter, sig_colcounter_next : unsigned(3 downto 0) := (others => '0');
signal sig_linecounter, sig_linecounter_next : unsigned(3 downto 0) := (others => '0');
signal sig_houghspace, sig_houghspace_next : hough_array := (others => (others => (others => '0')));

signal sig_y_temp0, sig_y_temp_next0 : signed(5 downto 0);
signal sig_x_temp0, sig_x_temp_next0 : signed(5 downto 0);
signal sig_y_temp1, sig_y_temp_next1 : signed(5 downto 0);
signal sig_x_temp1, sig_x_temp_next1 : signed(5 downto 0);
signal sig_y_temp2, sig_y_temp_next2 : signed(5 downto 0);
signal sig_x_temp2, sig_x_temp_next2 : signed(5 downto 0);
signal sig_y_temp3, sig_y_temp_next3 : signed(5 downto 0);
signal sig_x_temp3, sig_x_temp_next3 : signed(5 downto 0);
signal sig_y_temp4, sig_y_temp_next4 : signed(5 downto 0);
signal sig_x_temp4, sig_x_temp_next4 : signed(5 downto 0);
signal sig_y_temp5, sig_y_temp_next5 : signed(5 downto 0);
signal sig_x_temp5, sig_x_temp_next5 : signed(5 downto 0);
signal sig_y_temp6, sig_y_temp_next6 : signed(5 downto 0);
signal sig_x_temp6, sig_x_temp_next6 : signed(5 downto 0);

signal coord_ready, coord_ready_next : std_logic;

begin

	--Clocked process, section clkproc
	clk_proc : process(clk, reset)
	begin
		if (reset = '1') then
			sig_colcounter <= (others => '0');
			sig_linecounter <= (others => '0');
			sig_houghspace <= (others => (others => (others => '0')));

			sig_y_temp0 <= (others => '0');
			sig_x_temp0 <= (others => '0');
			sig_y_temp1 <= (others => '0');
			sig_x_temp1 <= (others => '0');
			sig_y_temp2 <= (others => '0');
			sig_x_temp2 <= (others => '0');
			sig_y_temp3 <= (others => '0');
			sig_x_temp3 <= (others => '0');
			sig_y_temp4 <= (others => '0');
			sig_x_temp4 <= (others => '0');
			sig_y_temp5 <= (others => '0');
			sig_x_temp5 <= (others => '0');
			sig_y_temp6 <= (others => '0');
			sig_x_temp6 <= (others => '0');

			coord_ready <= '0';
		elsif(clk'event and clk = '1') then
			if(enable = '1') then
				sig_colcounter <= sig_colcounter_next;
				sig_linecounter <= sig_linecounter_next;
				sig_houghspace <= sig_houghspace_next;

				sig_y_temp0 <= sig_y_temp_next0;
				sig_x_temp0 <= sig_x_temp_next0;
				sig_y_temp1 <= sig_y_temp_next1;
				sig_x_temp1 <= sig_x_temp_next1;
				sig_y_temp2 <= sig_y_temp_next2;
				sig_x_temp2 <= sig_x_temp_next2;
				sig_y_temp3 <= sig_y_temp_next3;
				sig_x_temp3 <= sig_x_temp_next3;
				sig_y_temp4 <= sig_y_temp_next4;
				sig_x_temp4 <= sig_x_temp_next4;
				sig_y_temp5 <= sig_y_temp_next5;
				sig_x_temp5 <= sig_x_temp_next5;
				sig_y_temp6 <= sig_y_temp_next6;
				sig_x_temp6 <= sig_x_temp_next6;

				coord_ready <= coord_ready_next;
			end if;
		end if;
	end process clk_proc;

	--Combinatoric process sensitive on input ports and current output port signals. Section "senslist"
	comb_proc_coord : process(
		sig_colcounter, 
		sig_linecounter, 
		serialIn, 
		coord_ready, 
		enable, 
		sig_houghspace, 
		sig_y_temp0, 
		sig_x_temp0, 
		sig_y_temp1, 
		sig_x_temp1, 
		sig_y_temp2, 
		sig_x_temp2, 
		sig_y_temp3, 
		sig_x_temp3, 
		sig_y_temp4, 
		sig_x_temp4, 
		sig_y_temp5, 
		sig_x_temp5, 
		sig_y_temp6, 
		sig_x_temp6
	)

	begin
		--Save current signals (register them)
		sig_colcounter_next <= sig_colcounter;
		sig_linecounter_next <= sig_linecounter;
		sig_houghspace_next <= sig_houghspace;

		coord_ready_next <= '0';	--reset this signal by default. It is only set in the process when coordinates are ready and voting can be done...

		sig_y_temp_next0 <= sig_y_temp0;
		sig_x_temp_next0 <= sig_x_temp0;
		sig_y_temp_next1 <= sig_y_temp1;
		sig_x_temp_next1 <= sig_x_temp1;
		sig_y_temp_next2 <= sig_y_temp2;
		sig_x_temp_next2 <= sig_x_temp2;
		sig_y_temp_next3 <= sig_y_temp3;
		sig_x_temp_next3 <= sig_x_temp3;
		sig_y_temp_next4 <= sig_y_temp4;
		sig_x_temp_next4 <= sig_x_temp4;
		sig_y_temp_next5 <= sig_y_temp5;
		sig_x_temp_next5 <= sig_x_temp5;
		sig_y_temp_next6 <= sig_y_temp6;
		sig_x_temp_next6 <= sig_x_temp6;

		--Count the positive clock edges at enable = 1, i.e. the pixels of the input image
		if (enable = '1') then
			if (sig_colcounter < IMAGE_WIDTH - 1) then
				sig_colcounter_next <= sig_colcounter + 1;
			else
				sig_colcounter_next <= (others => '0');
				if (sig_linecounter < IMAGE_HEIGHT - 1) then
					sig_linecounter_next <= sig_linecounter + 1;
				else
					sig_linecounter_next <= (others => '0');
				end if;
			end if;
		end if;

		--Check, if pixel is valid. If so, calculate Hough Transform coordinates to vote in Hough Space:

		if (serialIn = '1') then
			sig_y_temp_next0 <= to_signed(to_integer(sig_linecounter) + to_integer(SHAPE_DATA(0,0)),6);
			sig_x_temp_next0 <= to_signed(to_integer(sig_colcounter) + to_integer(SHAPE_DATA(0,1)),6);

			sig_y_temp_next1 <= to_signed(to_integer(sig_linecounter) + to_integer(SHAPE_DATA(1,0)),6);
			sig_x_temp_next1 <= to_signed(to_integer(sig_colcounter) + to_integer(SHAPE_DATA(1,1)),6);

			sig_y_temp_next2 <= to_signed(to_integer(sig_linecounter) + to_integer(SHAPE_DATA(2,0)),6);
			sig_x_temp_next2 <= to_signed(to_integer(sig_colcounter) + to_integer(SHAPE_DATA(2,1)),6);

			sig_y_temp_next3 <= to_signed(to_integer(sig_linecounter) + to_integer(SHAPE_DATA(3,0)),6);
			sig_x_temp_next3 <= to_signed(to_integer(sig_colcounter) + to_integer(SHAPE_DATA(3,1)),6);

			sig_y_temp_next4 <= to_signed(to_integer(sig_linecounter) + to_integer(SHAPE_DATA(4,0)),6);
			sig_x_temp_next4 <= to_signed(to_integer(sig_colcounter) + to_integer(SHAPE_DATA(4,1)),6);

			sig_y_temp_next5 <= to_signed(to_integer(sig_linecounter) + to_integer(SHAPE_DATA(5,0)),6);
			sig_x_temp_next5 <= to_signed(to_integer(sig_colcounter) + to_integer(SHAPE_DATA(5,1)),6);

			sig_y_temp_next6 <= to_signed(to_integer(sig_linecounter) + to_integer(SHAPE_DATA(6,0)),6);
			sig_x_temp_next6 <= to_signed(to_integer(sig_colcounter) + to_integer(SHAPE_DATA(6,1)),6);

			coord_ready_next <= '1';
		end if;

		if (coord_ready = '1') then
			if ((sig_y_temp0 >= 0) and (sig_y_temp0 <= HOUGH_HEIGHT - 1) and (sig_x_temp0 >= 0) and (sig_x_temp0 <= HOUGH_WIDTH - 1)) then
				sig_houghspace_next(to_integer(sig_y_temp0(4 downto 0)), to_integer(sig_x_temp0(4 downto 0))) <= std_logic_vector(unsigned(sig_houghspace(to_integer(sig_y_temp0(4 downto 0)), to_integer(sig_x_temp0(4 downto 0)))) + 1);
			end if;

			if ((sig_y_temp1 >= 0) and (sig_y_temp1 <= HOUGH_HEIGHT - 1) and (sig_x_temp1 >= 0) and (sig_x_temp1 <= HOUGH_WIDTH - 1)) then
				sig_houghspace_next(to_integer(sig_y_temp1(4 downto 0)), to_integer(sig_x_temp1(4 downto 0))) <= std_logic_vector(unsigned(sig_houghspace(to_integer(sig_y_temp1(4 downto 0)), to_integer(sig_x_temp1(4 downto 0)))) + 1);
			end if;

			if ((sig_y_temp2 >= 0) and (sig_y_temp2 <= HOUGH_HEIGHT - 1) and (sig_x_temp2 >= 0) and (sig_x_temp2 <= HOUGH_WIDTH - 1)) then
				sig_houghspace_next(to_integer(sig_y_temp2(4 downto 0)), to_integer(sig_x_temp2(4 downto 0))) <= std_logic_vector(unsigned(sig_houghspace(to_integer(sig_y_temp2(4 downto 0)), to_integer(sig_x_temp2(4 downto 0)))) + 1);
			end if;

			if ((sig_y_temp3 >= 0) and (sig_y_temp3 <= HOUGH_HEIGHT - 1) and (sig_x_temp3 >= 0) and (sig_x_temp3 <= HOUGH_WIDTH - 1)) then
				sig_houghspace_next(to_integer(sig_y_temp3(4 downto 0)), to_integer(sig_x_temp3(4 downto 0))) <= std_logic_vector(unsigned(sig_houghspace(to_integer(sig_y_temp3(4 downto 0)), to_integer(sig_x_temp3(4 downto 0)))) + 1);
			end if;

			if ((sig_y_temp4 >= 0) and (sig_y_temp4 <= HOUGH_HEIGHT - 1) and (sig_x_temp4 >= 0) and (sig_x_temp4 <= HOUGH_WIDTH - 1)) then
				sig_houghspace_next(to_integer(sig_y_temp4(4 downto 0)), to_integer(sig_x_temp4(4 downto 0))) <= std_logic_vector(unsigned(sig_houghspace(to_integer(sig_y_temp4(4 downto 0)), to_integer(sig_x_temp4(4 downto 0)))) + 1);
			end if;

			if ((sig_y_temp5 >= 0) and (sig_y_temp5 <= HOUGH_HEIGHT - 1) and (sig_x_temp5 >= 0) and (sig_x_temp5 <= HOUGH_WIDTH - 1)) then
				sig_houghspace_next(to_integer(sig_y_temp5(4 downto 0)), to_integer(sig_x_temp5(4 downto 0))) <= std_logic_vector(unsigned(sig_houghspace(to_integer(sig_y_temp5(4 downto 0)), to_integer(sig_x_temp5(4 downto 0)))) + 1);
			end if;

			if ((sig_y_temp6 >= 0) and (sig_y_temp6 <= HOUGH_HEIGHT - 1) and (sig_x_temp6 >= 0) and (sig_x_temp6 <= HOUGH_WIDTH - 1)) then
				sig_houghspace_next(to_integer(sig_y_temp6(4 downto 0)), to_integer(sig_x_temp6(4 downto 0))) <= std_logic_vector(unsigned(sig_houghspace(to_integer(sig_y_temp6(4 downto 0)), to_integer(sig_x_temp6(4 downto 0)))) + 1);
			end if;
		end if;
	end process comb_proc_coord;

	--Outputs: (Current output signals to module output ports. Must be outside of processes!):
	hough_space <= sig_houghspace;

end Behavioral;


