-------------------------------------------------
--file automatically generated

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

library work;
use work.testimage.ALL;
use work.maskdata.ALL;

--Top Module: 
ENTITY Houghtransform IS
	PORT(
		clk : in std_logic;
		reset : in std_logic;
		enable : in std_logic;
		pixIn : in std_logic;
		pixOut : out std_logic_vector(ValidMaskPixelsInBits - 1 downto 0); --Bit size generated from valid pixels in mask
		validMax : out std_logic
	);
END ENTITY Houghtransform;

ARCHITECTURE behavioral OF Houghtransform IS 

--CONSTANT DelayPipelineLength: natural := 15905;	--Generated, is ((maskheight-1) * imagewidth) + maskwidth + 1
CONSTANT DelayPipelineLength: natural := ((MaskHeight-1) * ImageWidth) + MaskWidth + 1;
TYPE DelayPipelineType IS ARRAY (natural range 0 to DelayPipelineLength - 1) OF unsigned(ValidMaskPixelsInBits - 1 downto 0);

SIGNAL delayPipeline, delayPipeline_next : DelayPipelineType;

BEGIN
Clocked_Proc : PROCESS(clk,reset)
BEGIN
	IF (reset = '1') THEN
		delayPipeline(0 TO DelayPipelineLength - 1) <= (OTHERS => (OTHERS => '0'));
		pixOut <= (OTHERS => '0');
		validMax <= '0';
	ELSIF (clk'event AND clk = '1') THEN
		IF (enable = '1') THEN
			pixOut <= std_logic_vector(delayPipeline(0));
			delayPipeline(0 TO DelayPipelineLength - 1) <= delayPipeline_next(0 TO DelayPipelineLength - 1);
			IF delayPipeline(0) = ValidMaskPixels THEN
				validMax <= '1';
			ELSE
				validMax <= '0';
			END IF;
		END IF;
	END IF; 
END PROCESS Clocked_Proc;

Comb_Proc : PROCESS(pixIn, delayPipeline)
BEGIN
	delayPipeline_next(0 TO DelayPipelineLength - 2) <= delayPipeline(1 TO DelayPipelineLength - 1);	--Shift pipeline. Voting will be done later;
	delayPipeline_next(DelayPipelineLength - 1) <= (OTHERS => '0');	--Shift zeros into the pipeline at the beginning

	IF (pixIn = '1') THEN
		FOR y in 0 to (MaskHeight - 1) LOOP
			FOR x in 0 to (MaskWidth - 1) LOOP
				IF (Mask(y,x) = '1') THEN
					delayPipeline_next(x + (y * ImageWidth)) <= delayPipeline(x + (y * ImageWidth) + 1) + 1;	--Vote in pipeline and shift
				END IF;
			END LOOP;
		END LOOP;
	END IF;
END PROCESS Comb_Proc;
END behavioral;