
--Testbench file

-------------------------------------------------
--file automatically generated

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

library work;
use work.testimage.ALL;
use work.maskdata.ALL;
use work.referencedata.ALL;

ENTITY testbench IS 
END testbench;

ARCHITECTURE behavioral OF testbench IS

COMPONENT Houghtransform
PORT( 
	clk : IN std_logic;
	reset : IN std_logic;
	enable : IN std_logic;
	pixIn : IN std_logic;
	pixOut : OUT std_logic_vector(ValidMaskPixelsInBits - 1 downto 0);
	validMax : OUT std_logic
);
END COMPONENT;

--Signals for input ports:
	signal clk : std_logic := '0';
	signal reset : std_logic := '0';
	signal enable: std_logic := '0';
	signal pixIn : std_logic := '0';

--Signal for output port:
	signal pixOut : std_logic_vector(ValidMaskPixelsInBits - 1 downto 0) := (others => '0');
	signal validMax : std_logic := '0';

--Constant for clock signal
	CONSTANT clk_period : time := 20 ns;

BEGIN

--Instantiate unit under test:
uut : Houghtransform PORT MAP(
	clk => clk,
	reset => reset,
	enable => enable,
	pixIn => pixIn,
	pixOut => pixOut,
	validMax => validMax
);

--Clock process:
clk_process : PROCESS
BEGIN
	clk <= '0';
	wait for clk_period / 2;
	clk <= '1';
	wait for clk_period / 2;
END PROCESS;

--Stimulus process
stim_proc : PROCESS
BEGIN
	wait for clk_period;
	reset <= '1';
	wait for clk_period;
	reset <= '0';
	wait for clk_period * 2;

	enable <= '1';

--Stimulus application
	FOR y in 0 to (ImageHeight - 1) LOOP
		FOR x in 0 to (ImageWidth - 1) LOOP
			pixIn <= IMAGE(y,x); 
			wait for clk_period;
		END LOOP;
	END LOOP;

	wait for clk_period;
	pixIn <= '0';	--Be sure to pull pixel in to low, even if last valid pixel was 1
	wait for clk_period; --Wait another cycle to let the hough module finish its work
	enable <= '0';

	wait; --forever
END PROCESS;

--Verify process:
verify_proc : PROCESS
	variable error : integer := 0;
	variable tempActual, tempRef : integer := 0;
BEGIN
	wait for clk_period * 6; --wait until reset occured

--Compare each received pixel with corresponding reference data
	FOR y in 0 TO (ImageHeight - 1) LOOP
		FOR x IN 0 TO (ImageWidth - 1) LOOP
			IF (pixOut /= to_integer(unsigned(HoughSpace(y,x)))) THEN
				error := error + 1;
				tempActual := to_integer(unsigned(pixOut));
				tempRef := to_integer(unsigned(HoughSpace(y,x)));
				report "ERROR at y=" & INTEGER'image(y) & ", and x= " & INTEGER'image(x) & " Should be: " & INTEGER'image(tempRef) & " but is: " & INTEGER'image(tempActual);
			END IF;
			IF validMax = '1' THEN
				report "Valid Max detected at current y=" & INTEGER'image(y) & ", and x= " & INTEGER'image(x) & " which is: " & INTEGER'image(to_integer(unsigned(pixOut)));
			END IF;
			wait for clk_period;
		END LOOP;
	END LOOP;

	--Terminate simulation and report success or failure:
	IF (error > 0) THEN
		report "Verification FAILED. In total, " & INTEGER'image(error) & " errors found." severity failure;
	ELSE
		report "Ignore the word failure. Verification SUCESSFULL!!!" severity failure;
	END IF;
	wait; --forever
END PROCESS verify_proc;

END; --Architecture




